#!/usr/bin/env python

"""This script performs Welch's t-test on a JSON export file with two
benchmark results to test whether or not the two distributions are
the same."""

import argparse
import json
import sys
from scipy import stats

parser = argparse.ArgumentParser(description=__doc__)
parser.add_argument("file", help="JSON file with two benchmark results")
args = parser.parse_args()

with open(args.file) as f:
    results = json.load(f)["results"]

if len(results) != 2:
    print("The input file has to contain exactly two benchmarks")
    sys.exit(1)

a, b = [x["command"] for x in results[:2]]
X, Y = [x["times"] for x in results[:2]]

print("Command 1: {}".format(a))
print("Command 2: {}\n".format(b))

t, p = stats.ttest_ind(X, Y, equal_var=False)
th = 0.05
dispose = p < th
print("t = {:.3}, p = {:.3}".format(t, p))
print()

if dispose:
    print("There is a difference between the two benchmarks (p < {}).".format(th))
else:
    print("The two benchmarks are almost the same (p >= {}).".format(th))



#!/usr/bin/env python

import argparse
import json
import numpy as np

parser = argparse.ArgumentParser()
parser.add_argument("file", help="JSON file with benchmark results")
args = parser.parse_args()

with open(args.file) as f:
    results = json.load(f)["results"]

commands = [b["command"] for b in results]
times = [b["times"] for b in results]

for command, ts in zip(commands, times):
    p05 = np.percentile(ts, 5)
    p25 = np.percentile(ts, 25)
    p75 = np.percentile(ts, 75)
    p95 = np.percentile(ts, 95)

    iqr = p75 - p25

    print("Command '{}'".format(command))
    print("  runs:   {:8d}".format(len(ts)))
    print("  mean:   {:8.3f} s".format(np.mean(ts)))
    print("  stddev: {:8.3f} s".format(np.std(ts, ddof=1)))
    print("  median: {:8.3f} s".format(np.median(ts)))
    print("  min:    {:8.3f} s".format(np.min(ts)))
    print("  max:    {:8.3f} s".format(np.max(ts)))
    print()
    print("  percentiles:")
    print("     P_05 .. P_95:    {:.3f} s .. {:.3f} s".format(p05, p95))
    print(
        "     P_25 .. P_75:    {:.3f} s .. {:.3f} s  "
        "(IQR = {:.3f} s)".format(p25, p75, iqr)
    )
    print()
